package polyglot.ext.ibex;

import java.util.Set;
import java.io.PrintStream;
import java.io.File;
import polyglot.main.Options;
import polyglot.main.UsageError;

/** 
 * This object encapsulates various polyglot options. 
 */
public class IbexOptions extends Options {
    /*
     * Fields for storing values for options.
     */

    /**
     * True if the grammar should be analyzed to check that all
     * necessary merge actions are implemented.
     */
    public boolean checkMergeActions;

    /** Constructor */
    public IbexOptions(ExtensionInfo extension) {
        super(extension);
    }

    /**
     * Set default values for options
     */
    public void setDefaultValues() {
        super.setDefaultValues();
        checkMergeActions = true;
    }
    
    /**
     * Parse a command
     * @return the next index to process. That is, if calling this method
     *         processes two commands, then the return value should be index+2
     */
    protected int parseCommand(String args[], int index, Set source) throws UsageError {
        if (args[index].equals("-mergecheck")) {
            index++;
            checkMergeActions = true;
        }
        else if (args[index].equals("-nomergecheck")) {
            index++;
            checkMergeActions = false;
        }
        else {
            int i = super.parseCommand(args, index, source);
            return i;
        }
        return index;
    }
    
    /**
     * Print usage information
     */
    public void usage(PrintStream out) {
        super.usage(out);
        usageForFlag(out, "-mergecheck", "check if all necessary merge " +
                                         "actions are implemented");
        usageForFlag(out, "-nomergecheck", "do not check if all necessary merge " +
                                         "actions are implemented");
    }
}
