package polyglot.ext.ibex.ast;

import polyglot.ast.*;
import polyglot.ext.jl.ast.*;
import polyglot.types.*;
import polyglot.ext.ibex.types.*;
import polyglot.ext.ibex.visit.*;
import polyglot.visit.*;
import polyglot.util.*;
import java.util.*;

/**
 * Tag interface for elements of a rule RHS.
 */
public class RhsMergeNode_c extends RhsNode_c implements RhsMergeNode {
    List elements1;
    List elements2;

    public RhsMergeNode_c(Position pos, List elements1, List elements2, Block action) {
        super(pos, action);
        this.elements1 = TypedList.copyAndCheck(elements1, RhsElement.class, true);
        this.elements2 = TypedList.copyAndCheck(elements2, RhsElement.class, true);
    }

    public List allElements() {
        List l = new ArrayList();
        l.addAll(elements1());
        l.addAll(elements2());
        return l;
    }

    public List elements1() { return elements1; }
    public RhsMergeNode elements1(List elements1) {
        RhsMergeNode_c n = (RhsMergeNode_c) copy();
        n.elements1 = TypedList.copyAndCheck(elements1, RhsElement.class, true);
        return n;
    }

    public List elements2() { return elements2; }
    public RhsMergeNode elements2(List elements2) {
        RhsMergeNode_c n = (RhsMergeNode_c) copy();
        n.elements2 = TypedList.copyAndCheck(elements2, RhsElement.class, true);
        return n;
    }

    protected RhsMergeNode_c reconstruct(List elements1, List elements2, Block action) {
        if (! CollectionUtil.equals(elements1, this.elements1) ||
            ! CollectionUtil.equals(elements2, this.elements2) ||
            action != this.action) {
            RhsMergeNode_c n = (RhsMergeNode_c) copy();
            n.elements1 = TypedList.copyAndCheck(elements1, RhsElement.class, true);
            n.elements2 = TypedList.copyAndCheck(elements2, RhsElement.class, true);
            n.action = action;
            return n;
        }
        return this;
    }

    public Node visitChildren(NodeVisitor v) {
        List elements1 = visitList(this.elements1, v);
        List elements2 = visitList(this.elements2, v);
        Block action = (Block) visitChild(this.action, v);
        return reconstruct(elements1, elements2, action);
    }

    protected Node makeRhs(IbexTypeSystem ts) {
        RhsCase rhs1 = makeRhsCase(ts, elements1);
        RhsCase rhs2 = makeRhsCase(ts, elements2);
        Rhs rhs = ts.rhsMerge(position(), rhs1, rhs2);
        return rhs(rhs);
    }

    protected RhsCase makeRhsCase(IbexTypeSystem ts, List elements) {
        List tags = new ArrayList(elements.size());
        List syms = new ArrayList(elements.size());

        List l = new ArrayList(elements.size());
        for (int i = 0; i < elements.size(); i++) {
            RhsElement n = (RhsElement) elements.get(i);
            if (n instanceof Tagged) {
                Tagged t = (Tagged) n;
                tags.add(t.tag());
            }
            else {
                tags.add(null);
            }

            syms.add(n.symbolInstance());
        }

        return ts.rhsCase(position(), syms, tags);
    }
}
