import ibex.runtime.*;

class Expr implements Mergeable {
    public Object merge(Object o) {
        System.out.println("Expr.merge(" + this + ", " + o + ") = " + this);
        return this;
    }
}
class Id extends Expr {
    String id;
    Id(String id) {
        this.id = id;
    }
    public String toString() {
        return id;
    }
}
class Plus extends Expr {
    Expr x, y;
    Plus(Expr x, Expr y) {
        this.x = x;
        this.y = y;
    }
    public String toString() {
        return "(" + x + " + " + y + ")";
    }
}
class Times extends Expr {
    Expr x, y;
    Times(Expr x, Expr y) {
        this.x = x;
        this.y = y;
    }
    public String toString() {
        return "(" + x + " * " + y + ")";
    }
    public Object merge(Object o) {
        // return the node with lower precedence
        if (o instanceof Plus) {
            System.out.println("Times.merge(" + this + ", " + o + ") = " + o);
            return o;
        }
        System.out.println("Times.merge(" + this + ", " + o + ") = " + this);
        return this;
    }
}

class List implements Mergeable {
  Expr head;
  List tail;
  List(Expr head, List tail) {
        this.head = head;
        this.tail = tail;
  }

  public String toString() {
        return head + ", " + tail;
  }

  public Object merge(Object o) {
        System.out.println("List.merge(" + this + ", " + o + ") = " + this);
        return this;
  }
}

class Token implements Terminal {
    int sym;
    Token(int sym) { this.sym = sym; }
    public int symbol() { return sym; }
    public String toString() { return "token" + sym; }
}

class IdToken extends Token {
    String value;
    IdToken(int sym, String x) { super(sym); this.value = x; }
    public String toString() { return value; }
}
