package polyglot.ext.polyj.ast;

import polyglot.ext.polyj.types.*;

import polyglot.ast.*;
import polyglot.types.*;
import polyglot.util.*;
import polyglot.visit.*;

import java.util.*;

public class ConstructorConstraint_c extends ProcedureConstraint_c implements ConstructorConstraint {
    
    public ConstructorConstraint_c(Position pos, 
                                        Flags flags, String name, List formals,
                                        List exceptions)
    {
        super(pos);
        this.flags = flags;
        this.name = name;
        this.formals = TypedList.copyAndCheck(formals, TypeNode.class, true);
        this.exceptions = TypedList.copyAndCheck(exceptions, TypeNode.class, true);
    }

    /**
     * Lazily reconstruct this node.
     */
    protected ConstructorConstraint reconstruct(List formals, List exceptions)
    {
        if(   !CollectionUtil.equals(this.formals, formals) 
           || !CollectionUtil.equals(this.exceptions, exceptions)) {
            ConstructorConstraint_c n = (ConstructorConstraint_c) copy();
            n.formals = TypedList.copyAndCheck(formals, TypeNode.class, true);
            n.exceptions = TypedList.copyAndCheck(exceptions, TypeNode.class, true);
            return n;
        } else {
            return this;
        }
    }

    public Node visitChildren(NodeVisitor v) 
    {
        List newFormals = visitList(formals, v);
        List newExceptions = visitList(exceptions, v);

        return reconstruct(newFormals, newExceptions);
    }

    public void prettyPrint(CodeWriter w, PrettyPrinter tr) {
        w.begin(0);
        w.write(flags.translate());
        w.write(name + "(");
        w.begin(0);

        for (Iterator i = formals.iterator(); i.hasNext();)
        {
            TypeNode tn = (TypeNode) i.next();
            print(tn, w, tr);
            if(i.hasNext()) {
                w.write(",");
                w.allowBreak(0, " ");
            }
        }
        w.end();
        w.write(")");

        if(!exceptions.isEmpty()) {
            w.allowBreak(6);
            w.write("throws ");
            for (Iterator i = exceptions.iterator(); i.hasNext();) {
                TypeNode tn = (TypeNode) i.next();
                print(tn, w, tr);
                if (i.hasNext()) {
                    w.write(",");
                    w.allowBreak(4, " ");
                }
            }
        }
        w.write(";");
        w.end();
    }

    public void dump(CodeWriter w) 
    {
        w.write("POLYJ CONSTRUCTOR CONSTRAINT");
        w.write(" < " + name + " >");
        w.write(" < " + flags.toString() + "> ");
    }

    /** 
     * Convert this into a ConstructorInstance associated with the given ParamType.
     */
    public ProcedureInstance toInstance(ParamType param) {
        List formalTypes = new ArrayList(formals.size());
        List throwTypes = new ArrayList(exceptions.size());

        for (Iterator it = formals.iterator(); it.hasNext(); ) {
            formalTypes.add((Type)((TypeNode)it.next()).type());
        }

        for (Iterator it = exceptions.iterator(); it.hasNext(); ) {
            throwTypes.add((Type)((TypeNode)it.next()).type());
        }

        Flags af = (Flags) flags.Public().Abstract();
        
        ConstructorInstance ci = param.typeSystem().constructorInstance(position(),
                                   param, af, formalTypes, throwTypes);
    
        return ci;
    }
}
